@@ -1,64 +1,171 @@
-// ser.rs
-// Copyright: (c) 2015-2021, Oleg Lelenkov
+//
+// Copyright (c) 2022 Oleg Lelenkov <o.lelenkov@gmail.com>
 // Distributed under terms of the BSD 3-Clause license.
 //
 
-use std::collections::HashMap;
 use std::result;
 use serde::ser;
 
-use crate::value::UniNode;
+use crate::value::{UniNode, Object, Array};
 use super::error::{Result, UniNodeSerError};
 
-#[cfg(test)]
-use {serde::Serialize, crate::unode, super::serialize, serde_bytes::Bytes};
+pub struct SerializeMap {
+    map: Object,
+    next_key: Option<String>,
+}
 
-impl ser::Serialize for UniNode {
-    fn serialize<S>(&self, serializer: S) -> result::Result<S::Ok, S::Error>
+impl ser::SerializeMap for SerializeMap {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_key<T>(&mut self, key: &T) -> Result<()>
     where
-        S: ser::Serializer,
+        T: ?Sized + ser::Serialize,
     {
-        match self {
-            UniNode::Boolean(v) => serializer.serialize_bool(*v),
-            UniNode::Integer(v) => serializer.serialize_i64(*v),
-            UniNode::UInteger(v) => serializer.serialize_u64(*v),
-            UniNode::Float(v) => serializer.serialize_f64(*v),
-            UniNode::Bytes(v) => serializer.serialize_bytes(v),
-            UniNode::String(v) => serializer.serialize_str(v),
-            UniNode::Null => serializer.serialize_unit(),
-            UniNode::Array(v) => v.serialize(serializer),
-            UniNode::Object(v) => {
-                use serde::ser::SerializeMap;
-                let mut map = serializer.serialize_map(Some(v.len()))?;
-                for (k, v) in v {
-                    map.serialize_entry(k, v)?;
-                }
-                map.end()
-            },
-        }
+        let key = key.serialize(UniNodeSerializer)?;
+        self.next_key = Some(
+            key.to_string()
+                .map_err(|_| UniNodeSerError::KeyMustBeAString)?,
+        );
+        Ok(())
+    }
+
+    fn serialize_value<T>(&mut self, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        let key = self
+            .next_key
+            .take()
+            .ok_or(UniNodeSerError::KeyMustBeAString)?;
+        self.map.insert(key, value.serialize(UniNodeSerializer)?);
+        Ok(())
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        Ok(UniNode::Object(self.map))
     }
 }
 
-#[test]
-fn serialize_trait_test() {
-    let unode = unode! {"one" => 1, "two" => 2, "arr" => unode![1, 2, 3]};
-    assert_eq!(serialize(&unode).unwrap(), unode);
-    #[derive(Debug, Serialize)]
-    struct Test {
-        val1: UniNode,
-        val2: UniNode,
+impl ser::SerializeStruct for SerializeMap {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        ser::SerializeMap::serialize_entry(self, key, value)
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        ser::SerializeMap::end(self)
     }
-    let data = Test {
-        val1: unode.clone(),
-        val2: unode!(1),
-    };
-    assert_eq!(
-        serialize(&data).unwrap(),
-        unode! {"val1" => unode, "val2" => 1}
-    );
 }
 
-#[derive(Debug, Default)]
+pub struct SerializeVec {
+    vec: Array,
+}
+
+impl ser::SerializeSeq for SerializeVec {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_element<T>(&mut self, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        self.vec.push(value.serialize(UniNodeSerializer)?);
+        Ok(())
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        Ok(UniNode::Array(self.vec))
+    }
+}
+
+impl ser::SerializeTuple for SerializeVec {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_element<T>(&mut self, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        ser::SerializeSeq::serialize_element(self, value)
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        ser::SerializeSeq::end(self)
+    }
+}
+
+impl ser::SerializeTupleStruct for SerializeVec {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_field<T>(&mut self, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        ser::SerializeSeq::serialize_element(self, value)
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        ser::SerializeSeq::end(self)
+    }
+}
+
+pub struct SerializeStructVariant {
+    map: Object,
+    name: String,
+}
+
+impl ser::SerializeStructVariant for SerializeStructVariant {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        self.map
+            .insert(String::from(key), value.serialize(UniNodeSerializer)?);
+        Ok(())
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        let mut tbl = Object::new();
+        tbl.insert(self.name, UniNode::Object(self.map));
+        Ok(UniNode::Object(tbl))
+    }
+}
+
+pub struct SerializeTupleVariant {
+    vec: Array,
+    name: String,
+}
+
+impl ser::SerializeTupleVariant for SerializeTupleVariant {
+    type Error = UniNodeSerError;
+    type Ok = UniNode;
+
+    fn serialize_field<T>(&mut self, value: &T) -> Result<()>
+    where
+        T: ?Sized + ser::Serialize,
+    {
+        self.vec.push(value.serialize(UniNodeSerializer)?);
+        Ok(())
+    }
+
+    fn end(self) -> Result<Self::Ok> {
+        let mut tbl = Object::new();
+        tbl.insert(self.name.to_string(), UniNode::Array(self.vec));
+        Ok(UniNode::Object(tbl))
+    }
+}
+
+#[derive(Default)]
 pub struct UniNodeSerializer;
 
 impl ser::Serializer for UniNodeSerializer {
@@ -168,7 +275,7 @@ impl ser::Serializer for UniNodeSerializer {
     where
         T: ?Sized + ser::Serialize,
     {
-        let mut ret = HashMap::new();
+        let mut ret = Object::new();
         ret.insert(var.to_string(), value.serialize(self)?);
         Ok(UniNode::Object(ret))
     }
@@ -194,13 +301,13 @@ impl ser::Serializer for UniNodeSerializer {
     ) -> Result<Self::SerializeTupleVariant> {
         Ok(SerializeTupleVariant {
             name: var.to_string(),
-            vec: Vec::with_capacity(len),
+            vec: Array::with_capacity(len),
         })
     }
 
     fn serialize_map(self, _len: Option<usize>) -> Result<Self::SerializeMap> {
         Ok(SerializeMap {
-            map: HashMap::new(),
+            map: Object::new(),
             next_key: None,
         })
     }
@@ -215,7 +322,7 @@ impl ser::Serializer for UniNodeSerializer {
         self, _name: &'static str, _idx: u32, var: &'static str, _len: usize,
     ) -> Result<Self::SerializeStructVariant> {
         Ok(SerializeStructVariant {
-            map: HashMap::new(),
+            map: Object::new(),
             name: var.to_string(),
         })
     }
@@ -228,307 +335,260 @@ impl ser::Serializer for UniNodeSerializer {
     }
 }
 
-#[test]
-fn serialize_primitive_test() {
-    use std::ops::RangeBounds;
-
-    assert_eq!(serialize(&true).unwrap(), unode!(true));
-    assert_eq!(serialize(&4u8).unwrap(), unode!(4u8));
-    assert_eq!(serialize(&4u16).unwrap(), unode!(4u16));
-    assert_eq!(serialize(&4u32).unwrap(), unode!(4u32));
-    assert_eq!(serialize(&4u64).unwrap(), unode!(4u64));
-    assert_eq!(serialize(&4i8).unwrap(), unode!(4i8));
-    assert_eq!(serialize(&4i16).unwrap(), unode!(4i16));
-    assert_eq!(serialize(&4i32).unwrap(), unode!(4i32));
-    assert_eq!(serialize(&4i64).unwrap(), unode!(4i64));
-    assert_eq!(serialize(&4.2f32).unwrap(), unode!(4.2f32));
-    assert_eq!(serialize(&43.3f64).unwrap(), unode!(43.3f64));
-    assert_eq!(serialize(&'a').unwrap(), unode!("a"));
-    assert_eq!(serialize(&"hello").unwrap(), unode!("hello"));
-    assert_eq!(serialize(&Option::<u8>::None).unwrap(), unode!());
-    assert_eq!(serialize(&Some(12)).unwrap(), unode!(12));
-    let bytes = Bytes::new(b"abc");
-    assert_eq!(serialize(&bytes).unwrap(), unode!(vec![97u8, 98u8, 99u8]));
-    assert_eq!(serialize(&()).unwrap(), unode!());
-    assert_eq!(
-        serialize(&std::marker::PhantomData::<u8>).unwrap(),
-        unode!()
-    );
-    assert_eq!(
-        serialize(&(..100).start_bound()).unwrap(),
-        unode!("Unbounded")
-    );
-}
-
-#[test]
-fn serialize_newtype_test() {
-    #[derive(Debug, Serialize)]
-    struct NewTypeStruct(u8);
-    assert_eq!(serialize(&NewTypeStruct(2)).unwrap(), unode!(2u8));
-
-    #[derive(Debug, Serialize)]
-    enum NewTypeEnum {
-        V(u8),
-    }
-    assert_eq!(serialize(&NewTypeEnum::V(4)).unwrap(), unode!("V" => 4u8));
-}
-
-pub struct SerializeVec {
-    vec: Vec<UniNode>,
-}
-
-impl ser::SerializeSeq for SerializeVec {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
-
-    fn serialize_element<T>(&mut self, value: &T) -> Result<()>
+impl ser::Serialize for UniNode {
+    fn serialize<S>(&self, serializer: S) -> result::Result<S::Ok, S::Error>
     where
-        T: ?Sized + ser::Serialize,
+        S: ser::Serializer,
     {
-        self.vec.push(value.serialize(UniNodeSerializer)?);
-        Ok(())
-    }
-
-    fn end(self) -> Result<Self::Ok> {
-        Ok(UniNode::Array(self.vec))
-    }
-}
-
-#[test]
-fn serialize_seq_test() {
-    let arr = vec!["one", "two", "three"];
-    assert_eq!(serialize(&arr).unwrap(), unode!("one", "two", "three"));
-}
-
-impl ser::SerializeTuple for SerializeVec {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
-
-    fn serialize_element<T>(&mut self, value: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        ser::SerializeSeq::serialize_element(self, value)
-    }
-
-    fn end(self) -> Result<Self::Ok> {
-        ser::SerializeSeq::end(self)
-    }
-}
-
-#[test]
-fn serialize_tuple_test() {
-    let tup = ("one", 1, 3.18);
-    assert_eq!(serialize(&tup).unwrap(), unode!("one", 1, 3.18));
-}
-
-impl ser::SerializeTupleStruct for SerializeVec {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
-
-    fn serialize_field<T>(&mut self, value: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        ser::SerializeSeq::serialize_element(self, value)
-    }
-
-    fn end(self) -> Result<Self::Ok> {
-        ser::SerializeSeq::end(self)
-    }
-}
-
-#[test]
-fn serialize_tuple_struct_test() {
-    #[derive(Serialize)]
-    struct Rgb(u8, u8, u8);
-
-    let tup = Rgb(2, 3, 4);
-    assert_eq!(serialize(&tup).unwrap(), unode!(2u8, 3u8, 4u8));
-}
-
-pub struct SerializeTupleVariant {
-    vec: Vec<UniNode>,
-    name: String,
-}
-
-impl ser::SerializeTupleVariant for SerializeTupleVariant {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
-
-    fn serialize_field<T>(&mut self, value: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        self.vec.push(value.serialize(UniNodeSerializer)?);
-        Ok(())
-    }
-
-    fn end(self) -> Result<Self::Ok> {
-        let mut tbl = HashMap::new();
-        tbl.insert(self.name.to_string(), UniNode::Array(self.vec));
-        Ok(UniNode::Object(tbl))
-    }
-}
-
-#[test]
-fn serialize_tuple_variant_test() {
-    #[derive(Serialize)]
-    enum E {
-        T(u8, u8),
-    }
-    let tup = E::T(2, 3);
-    assert_eq!(serialize(&tup).unwrap(), unode!("T" => unode![2u8, 3u8]));
-}
-
-pub struct SerializeMap {
-    map: HashMap<String, UniNode>,
-    next_key: Option<String>,
-}
-
-impl ser::SerializeMap for SerializeMap {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
-
-    fn serialize_key<T>(&mut self, key: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        let key = key.serialize(UniNodeSerializer)?;
-        self.next_key = Some(
-            key.to_string()
-                .map_err(|_| UniNodeSerError::KeyMustBeAString)?,
-        );
-        Ok(())
-    }
-
-    fn serialize_value<T>(&mut self, value: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        let key = self
-            .next_key
-            .take()
-            .ok_or(UniNodeSerError::KeyMustBeAString)?;
-        self.map.insert(key, value.serialize(UniNodeSerializer)?);
-        Ok(())
-    }
-
-    fn end(self) -> Result<Self::Ok> {
-        Ok(UniNode::Object(self.map))
-    }
-}
-
-#[test]
-fn serialize_map_test() {
-    #[derive(PartialEq, Eq, Hash, Serialize)]
-    struct Key(u8);
-    let mut map = HashMap::new();
-    map.insert(Key(1), "One");
-    map.insert(Key(2), "Two");
-    map.insert(Key(3), "Three");
-    assert_eq!(
-        serialize(&map).unwrap(),
-        unode! {"1" => "One", "2" => "Two", "3" => "Three"}
-    );
-}
-
-impl ser::SerializeStruct for SerializeMap {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
-
-    fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        ser::SerializeMap::serialize_entry(self, key, value)
-    }
-
-    fn end(self) -> Result<Self::Ok> {
-        ser::SerializeMap::end(self)
-    }
-}
-
-#[test]
-fn serialize_struct_test() {
-    #[derive(Debug, Serialize)]
-    struct Point {
-        x: i32,
-        y: i32,
-    }
-
-    impl Point {
-        fn new(x: i32, y: i32) -> Self {
-            Self { x, y }
+        match self {
+            UniNode::Boolean(v) => serializer.serialize_bool(*v),
+            UniNode::Integer(v) => serializer.serialize_i64(*v),
+            UniNode::UInteger(v) => serializer.serialize_u64(*v),
+            UniNode::Float(v) => serializer.serialize_f64(*v),
+            UniNode::Bytes(v) => serializer.serialize_bytes(v),
+            UniNode::String(v) => serializer.serialize_str(v),
+            UniNode::Null => serializer.serialize_unit(),
+            UniNode::Array(v) => v.serialize(serializer),
+            UniNode::Object(v) => {
+                use serde::ser::SerializeMap;
+                let mut map = serializer.serialize_map(Some(v.len()))?;
+                for (k, v) in v {
+                    map.serialize_entry(k, v)?;
+                }
+                map.end()
+            },
         }
     }
-
-    #[derive(Debug, Serialize)]
-    struct Test<'a> {
-        int: u32,
-        float: f64,
-        text: String,
-        data: Vec<Point>,
-        node: UniNode,
-        bytes: &'a Bytes,
-    }
-
-    let data = Test {
-        int: 42,
-        float: 3.18,
-        text: "hello".to_string(),
-        data: vec![Point::new(1, 2), Point::new(2, 3), Point::new(3, 4)],
-        node: 10.into(),
-        bytes: Bytes::new(b"abc"),
-    };
-
-    assert_eq!(serialize(&data).unwrap(), unode! {
-        "int" => 42u32,
-        "float" => 3.18,
-        "text" => "hello",
-        "data" => unode![
-            unode!{"x" => 1, "y" => 2},
-            unode!{"x" => 2, "y" => 3},
-            unode!{"x" => 3, "y" => 4},
-        ],
-        "node" => 10,
-        "bytes" => vec![97u8, 98u8, 99u8],
-    });
 }
 
-pub struct SerializeStructVariant {
-    map: HashMap<String, UniNode>,
-    name: String,
+pub fn serialize<T>(value: &T) -> Result<UniNode>
+where
+    T: serde::Serialize,
+{
+    value.serialize(UniNodeSerializer)
 }
 
-impl ser::SerializeStructVariant for SerializeStructVariant {
-    type Error = UniNodeSerError;
-    type Ok = UniNode;
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use crate::unode;
+    use serde::Serialize;
+    use serde_bytes::Bytes;
 
-    fn serialize_field<T>(&mut self, key: &'static str, value: &T) -> Result<()>
-    where
-        T: ?Sized + ser::Serialize,
-    {
-        self.map
-            .insert(String::from(key), value.serialize(UniNodeSerializer)?);
-        Ok(())
+    #[test]
+    fn serialize_node() {
+        let unode = unode! {"one" => 1, "two" => 2, "arr" => unode![1, 2, 3]};
+        assert_eq!(serialize(&unode).unwrap(), unode);
+
+        #[derive(Debug, Serialize)]
+        struct Test {
+            val1: UniNode,
+            val2: UniNode,
+        }
+        let data = Test {
+            val1: unode.clone(),
+            val2: unode!(1),
+        };
+        assert_eq!(
+            serialize(&data).unwrap(),
+            unode! {"val1" => unode, "val2" => 1}
+        );
     }
 
-    fn end(self) -> Result<Self::Ok> {
-        let mut tbl = HashMap::new();
-        tbl.insert(self.name, UniNode::Object(self.map));
-        Ok(UniNode::Object(tbl))
+    #[test]
+    fn serialize_primitives() {
+        use std::ops::RangeBounds;
+
+        assert_eq!(serialize(&true).unwrap(), unode!(true));
+        assert_eq!(serialize(&4u8).unwrap(), unode!(4u8));
+        assert_eq!(serialize(&4u16).unwrap(), unode!(4u16));
+        assert_eq!(serialize(&4u32).unwrap(), unode!(4u32));
+        assert_eq!(serialize(&4u64).unwrap(), unode!(4u64));
+        assert_eq!(serialize(&4i8).unwrap(), unode!(4i8));
+        assert_eq!(serialize(&4i16).unwrap(), unode!(4i16));
+        assert_eq!(serialize(&4i32).unwrap(), unode!(4i32));
+        assert_eq!(serialize(&4i64).unwrap(), unode!(4i64));
+        assert_eq!(serialize(&4.2f32).unwrap(), unode!(4.2f32));
+        assert_eq!(serialize(&43.3f64).unwrap(), unode!(43.3f64));
+        assert_eq!(serialize(&'a').unwrap(), unode!("a"));
+        assert_eq!(serialize(&"hello").unwrap(), unode!("hello"));
+        assert_eq!(serialize(&Option::<u8>::None).unwrap(), unode!());
+        assert_eq!(serialize(&Some(12)).unwrap(), unode!(12));
+        let bytes = Bytes::new(b"abc");
+        assert_eq!(serialize(&bytes).unwrap(), unode!(vec![97u8, 98u8, 99u8]));
+        assert_eq!(serialize(&()).unwrap(), unode!());
+        assert_eq!(
+            serialize(&std::marker::PhantomData::<u8>).unwrap(),
+            unode!()
+        );
+        assert_eq!(
+            serialize(&(..100).start_bound()).unwrap(),
+            unode!("Unbounded")
+        );
+    }
+
+    #[test]
+    fn serialize_newtype() {
+        #[derive(Debug, Serialize)]
+        struct NewTypeStruct(u8);
+        assert_eq!(serialize(&NewTypeStruct(2)).unwrap(), unode!(2u8));
+
+        #[derive(Debug, Serialize)]
+        enum NewTypeEnum {
+            V(u8),
+        }
+        assert_eq!(serialize(&NewTypeEnum::V(4)).unwrap(), unode!("V" => 4u8));
+    }
+
+    #[test]
+    fn serialize_seq() {
+        let arr = vec!["one", "two", "three"];
+        assert_eq!(serialize(&arr).unwrap(), unode!("one", "two", "three"));
+    }
+
+    #[test]
+    fn serialize_tuple() {
+        let tup = ("one", 1, 3.18);
+        assert_eq!(serialize(&tup).unwrap(), unode!("one", 1, 3.18));
+    }
+
+    #[test]
+    fn serialize_tuple_struct() {
+        #[derive(Serialize)]
+        struct Rgb(u8, u8, u8);
+
+        let tup = Rgb(2, 3, 4);
+        assert_eq!(serialize(&tup).unwrap(), unode!(2u8, 3u8, 4u8));
+    }
+
+    #[test]
+    fn serialize_tuple_variant() {
+        #[derive(Serialize)]
+        enum E {
+            T(u8, u8),
+        }
+        let tup = E::T(2, 3);
+        assert_eq!(serialize(&tup).unwrap(), unode!("T" => unode![2u8, 3u8]));
+    }
+
+    #[test]
+    fn serialize_struct() {
+        #[derive(Debug, Serialize)]
+        struct Point {
+            x: i32,
+            y: i32,
+        }
+
+        impl Point {
+            fn new(x: i32, y: i32) -> Self {
+                Self { x, y }
+            }
+        }
+
+        #[derive(Debug, Serialize)]
+        struct Test<'a> {
+            int: u32,
+            float: f64,
+            text: String,
+            data: Vec<Point>,
+            node: UniNode,
+            bytes: &'a Bytes,
+        }
+
+        let data = Test {
+            int: 42,
+            float: 3.18,
+            text: "hello".to_string(),
+            data: vec![Point::new(1, 2), Point::new(2, 3), Point::new(3, 4)],
+            node: 10.into(),
+            bytes: Bytes::new(b"abc"),
+        };
+
+        assert_eq!(
+            serialize(&data).unwrap(),
+            unode! {
+                "int" => 42u32,
+                "float" => 3.18,
+                "text" => "hello",
+                "data" => unode![
+                    unode!{"x" => 1, "y" => 2},
+                    unode!{"x" => 2, "y" => 3},
+                    unode!{"x" => 3, "y" => 4},
+                ],
+                "node" => 10,
+                "bytes" => vec![97u8, 98u8, 99u8],
+            }
+        );
+    }
+
+    #[test]
+    fn serialize_map() {
+        use std::collections::HashMap;
+
+        #[derive(PartialEq, Eq, Hash, Serialize)]
+        struct Key(u8);
+        let mut map = HashMap::new();
+        map.insert(Key(1), "One");
+        map.insert(Key(2), "Two");
+        map.insert(Key(3), "Three");
+        assert_eq!(
+            serialize(&map).unwrap(),
+            unode! {"1" => "One", "2" => "Two", "3" => "Three"}
+        );
+    }
+
+    #[test]
+    fn serialize_struct_variant() {
+        #[derive(Serialize)]
+        enum E {
+            S { r: u8, g: u8, b: u8 },
+        }
+        let data = E::S { r: 3, g: 5, b: 6 };
+        assert_eq!(
+            serialize(&data).unwrap(),
+            unode! {
+                "S" => unode!{"r" => 3u8, "g" => 5u8, "b" => 6u8}
+            }
+        );
+    }
+
+    #[test]
+    fn serialize_bytes() {
+        use serde::{Serialize, Deserialize, de::DeserializeOwned};
+        use std::fmt::Debug;
+
+        fn test_owned_data<T>(data: T)
+        where
+            T: Serialize + DeserializeOwned + Debug + PartialEq,
+        {
+            let node = serialize(&data).unwrap();
+            assert_eq!(T::deserialize(&node).unwrap(), data);
+            assert_eq!(T::deserialize(node).unwrap(), data);
+        }
+
+        test_owned_data(vec![1, 2, 3]);
+        test_owned_data(String::from("hello"));
+
+        let data = "abc";
+        let node = serialize(&data).unwrap();
+        assert_eq!(<&str>::deserialize(&node).unwrap(), data);
+
+        let data = Bytes::new(b"abc");
+        let node = serialize(&data).unwrap();
+        assert_eq!(<&Bytes>::deserialize(&node).unwrap(), data);
+
+        #[derive(Debug, PartialEq, Serialize, Deserialize)]
+        struct Data<'a> {
+            #[serde(with = "serde_bytes")]
+            data_slice: &'a [u8],
+            #[serde(with = "serde_bytes")]
+            data_buf: Vec<u8>,
+        }
+        let data = Data {
+            data_slice: b"abc",
+            data_buf: vec![1, 2, 3],
+        };
+        let node = serialize(&data).unwrap();
+        assert_eq!(Data::deserialize(&node).unwrap(), data);
     }
 }
-
-#[test]
-fn serialize_struct_variant_test() {
-    #[derive(Serialize)]
-    enum E {
-        S { r: u8, g: u8, b: u8 },
-    }
-    let data = E::S { r: 3, g: 5, b: 6 };
-    assert_eq!(serialize(&data).unwrap(), unode! {
-        "S" => unode!{"r" => 3u8, "g" => 5u8, "b" => 6u8}
-    });
-}
